﻿Imports System
Imports System.Windows.Forms
Imports AddinExpress.MSO

'Add-in Express Outlook Item Events Class
Public Class OutlookItemEvents
    Inherits AddinExpress.MSO.ADXOutlookItemEvents
 
    Public Sub New(ByVal ADXModule As AddinExpress.MSO.ADXAddinModule)
        MyBase.New(ADXModule)
    End Sub

    Private Function getSubject() As String
        Dim subject As String = ""
        If Me.IsConnected Then
            ' use late binding to retrieve the Subject property of the item
            Dim obj As Object = _
                Me.ItemObj.GetType().InvokeMember("Subject", System.Reflection.BindingFlags.GetProperty, Nothing, Me.ItemObj, Nothing)
            If obj IsNot Nothing Then subject = obj.ToString()
        End If
        Return "'" & subject & "'"
    End Function
 
    ''' <summary>
    ''' Is called when the AttachmentAdd event is generated by the Outlook item to which this class is connected. The event
    ''' occurs when an attachment has been added to the item.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) that was added to the item.</param>
    Public Overrides Sub ProcessAttachmentAdd(ByVal attachment As Object)
        Console.WriteLine("The event AttachmentAdd of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the AttachmentRead event is generated by the Outlook item to which this class is connected. The event
    ''' occurs when an attachment in the item has been opened for reading.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) that was opened for reading.</param>
    Public Overrides Sub ProcessAttachmentRead(ByVal attachment As Object)
        Console.WriteLine("The event AttachmentRead of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAttachmentSave event is generated by the Outlook item to which this class is connected. The
    ''' event occurs just before an attachment is saved.
    ''' </summary>
    ''' <remarks>
    ''' This event corresponds to when attachments are saved to the messaging store. The BeforeAttachmentSave event occurs just
    ''' before an attachment is saved when an item is saved. If a user edits an attachment and then saves those changes, the
    ''' BeforeAttachmentSave event will not occur at that time; instead it will occur when the item itself is later saved. It
    ''' also does not occur when the attachment is saved on the hard disk using the SaveAsFile method. 
    ''' <para></para>
    ''' <para>Add-in Express relies on the state of the COM object provided in this class member. That's why you must never
    ''' release it. On the contrary, you must release every COM object obtained from this COM object. </para>
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) to be saved.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAttachmentSave(ByVal attachment As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAttachmentSave of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeCheckNames event is generated by the Outlook item to which this class is connected. The event
    ''' occurs just before Outlook starts resolving names in the recipient collection for the item.
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeCheckNames(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeCheckNames of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Close event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the Inspector associated with the Outlook item is being closed.
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessClose(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The Close event of the item has occured. Do not access item's properties in this event!")
        Console.WriteLine("Disconnecting from the item")
        Me.RemoveConnection()
    End Sub

    ''' <summary>
    ''' Is called when the CustomAction event is generated by the Outlook item to which this class is connected. The event
    ''' occurs when a custom action of the Outlook item executes. The Action object and the newly-created item resulting from
    ''' the custom action are passed to the event.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="action">The Action object (a COM object) which represents the custom action that executes.</param>
    ''' <param name="response">The newly created item (a COM object) resulting from the custom action.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the custom
    ''' action is not completed.</param>
    Public Overrides Sub ProcessCustomAction(ByVal action As Object, ByVal response As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event CustomAction of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the CustomPropertyChange event is generated by the Outlook item to which this class is connected. The
    ''' event occurs when a custom property of the Outlook item is changed. The property name is passed to the procedure so that
    ''' you can determine which custom property changed.
    ''' </summary>
    ''' <param name="name">The name of the custom property that was changed.</param>
    Public Overrides Sub ProcessCustomPropertyChange(ByVal name As String)
        Console.WriteLine("The event CustomPropertyChange of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Forward event is generated by the Outlook item to which this class is connected. The event occurs
    ''' when the user selects the Forward action for the Outlook item.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="forward">The new item (a COM object) being forwarded.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the forward
    ''' operation is not completed and the new item is not displayed.</param>
    Public Overrides Sub ProcessForward(ByVal forward As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event Forward of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Open event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the Outlook item is being opened in an Inspector. When this event occurs, the Inspector object is initialized but not
    ''' yet displayed. The Open event differs from the Read event in that Read occurs whenever the user selects the item in a
    ''' view that supports in-cell editing as well as when the item is being opened in an Inspector.
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the open
    ''' operation is not completed and the inspector is not displayed.</param>
    Public Overrides Sub ProcessOpen(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event Open of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the PropertyChange event is generated by the Outlook item to which this class is connected. The event
    ''' occurs when a standard property (for example, Subject or To) of the Outlook item is changed. The property name is passed
    ''' to the event so that you can determine which property was changed.
    ''' </summary>
    ''' <param name="name">The name of the property that was changed.</param>
    Public Overrides Sub ProcessPropertyChange(ByVal name As String)
        Console.WriteLine("The event PropertyChange of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Read event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the Outlook item is opened for editing by the user. The Read event differs from the Open event in that Read occurs
    ''' whenever the user selects the item in a view that supports in-cell editing as well as when the item is being opened in
    ''' an Inspector.
    ''' </summary>
    Public Overrides Sub ProcessRead()
        Console.WriteLine("The event Read of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Reply event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the user selects the Reply action for the Outlook item.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="response">The new item (a COM object) being sent in response to the original message.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the reply
    ''' operation is not completed and the new item is not displayed.</param>
    Public Overrides Sub ProcessReply(ByVal response As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event Reply of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the ReplyAll event is generated by the Outlook item to which this class is connected. The event occurs
    ''' when the user selects the ReplyAll action for the Outlook item.
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="response">The new item (a COM object) being sent in response to the original message.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the reply
    ''' operation is not completed and the new item is not displayed.</param>
    Public Overrides Sub ProcessReplyAll(ByVal response As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event ReplyAll of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Send event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the user selects the Send action for the Outlook item.
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the open
    ''' operation is not completed and the inspector is left open.</param>
    Public Overrides Sub ProcessSend(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event Send of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Write event is generated by the Outlook item to which this class is connected. The event occurs when
    ''' the Outlook item is saved, either explicitly (for example, using the Save or SaveAs methods) or implicitly (for example,
    ''' in response to a prompt when closing the item's inspector).
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the save
    ''' operation is not completed.</param>
    Public Overrides Sub ProcessWrite(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event Write of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeDelete event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2002. It occurs before the item is deleted.</para>
    ''' </summary>
    ''' <remarks>
    ''' In order for this event to fire when an e-mail message, distribution list, journal entry, task, contact, or post are
    ''' deleted through an action, an inspector must be open. 
    ''' <para></para>
    ''' <para>Add-in Express relies on the state of the COM object provided in this class member. That's why you must never
    ''' release it. On the contrary, you must release every COM object obtained from this COM object. </para>
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="item">The item (a COM object) being deleted.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation. If e.Cancel is set to true, the reply
    ''' operation is not completed and the new item is not displayed.</param>
    Public Overrides Sub ProcessBeforeDelete(ByVal item As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeDelete of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the AttachmentRemove event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs when an attachment has been removed from an instance of the
    ''' parent object.</para>
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) that was removed from the item.</param>
    Public Overrides Sub ProcessAttachmentRemove(ByVal attachment As Object)
        Console.WriteLine("The event AttachmentRemove of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAttachmentAdd event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before an attachment is added to an instance of the parent
    ''' object.</para>
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) to be added to the item.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAttachmentAdd(ByVal attachment As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAttachmentAdd of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAttachmentPreview event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before an attachment associated with an instance of the parent
    ''' object is previewed.</para>
    ''' </summary>
    ''' <remarks>
    ''' This event occurs before an attachment is previewed, either from the attachment strip in the Reading Pane of the active
    ''' explorer or from the active inspector. 
    ''' <para></para>
    ''' <para>Add-in Express relies on the state of the COM object provided in this class member. That's why you must never
    ''' release it. On the contrary, you must release every COM object obtained from this COM object. </para>
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) to be previewed.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAttachmentPreview(ByVal attachment As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAttachmentPreview of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAttachmentRead event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before an attachment associated with an instance of the parent
    ''' object is read from the file system, an attachment stream, or an Attachment object.</para>
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) to be read.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAttachmentRead(ByVal attachment As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAttachmentRead of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAttachmentWriteToTempFile event is generated by the Outlook item to which this class is
    ''' connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before an attachment associated with an instance of the parent
    ''' object is written to a temporary file.</para>
    ''' </summary>
    ''' <remarks>
    ''' Add-in Express relies on the state of the COM object provided in this class member. That's why you must never release
    ''' it. On the contrary, you must release every COM object obtained from this COM object. 
    ''' <para></para>
    ''' <para>To release a COM object, you use the System.Runtime.InteropServices.Marshal.ReleaseComObject method.</para>
    ''' </remarks>
    ''' <param name="attachment">The attachment (a COM object) to be written.</param>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAttachmentWriteToTempFile(ByVal attachment As Object, ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAttachmentWriteToTempFile of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the Unload event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before the Outlook item is unloaded from memory, either
    ''' programmatically or by user action.</para>
    ''' </summary>
    ''' <remarks>
    ''' This event occurs after the Close event for the Outlook item occurs, but before the Outlook item is unloaded from
    ''' memory, allowing an add-in to release any resources related to the object. Although the event occurs before the Outlook
    ''' item is unloaded from memory, this event cannot be canceled.
    ''' </remarks>
    Public Overrides Sub ProcessUnload()
        Console.WriteLine("The event Unload of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the BeforeAutoSave event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2007. It occurs before the item is automatically saved by Outlook.</para>
    ''' </summary>
    ''' <param name="e">An ADXCancelEventArgs that allows cancelling the operation.</param>
    Public Overrides Sub ProcessBeforeAutoSave(ByVal e As AddinExpress.MSO.ADXCancelEventArgs)
        Console.WriteLine("The event BeforeAutoSave of the item with the subject " & getSubject() & " has occured.")
    End Sub


    ''' <summary>
    ''' Is called when the BeforeRead event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2010. It occurs before Outlook begins to read the properties for the
    ''' item.</para>
    ''' </summary>
    ''' <remarks>
    ''' The BeforeRead event occurs before the Read event. Unlike other events with the Before prefix, this event is not
    ''' cancelable. To determine when the item is unloaded from memory, use the Unload event.
    ''' </remarks>
    Public Overrides Sub ProcessBeforeRead()
        Console.WriteLine("The event BeforeRead of the item with the subject " & getSubject() & " has occured.")
    End Sub

    ''' <summary>
    ''' Is called when the AfterWrite event is generated by the Outlook item to which this class is connected. 
    ''' <para>The event was introduced in Outlook 2010. It occurs after Outlook has saved the item.</para>
    ''' </summary>
    ''' <remarks>
    ''' The AfterWrite event occurs after the Write event. This event is not cancelable. To determine when the item is unloaded
    ''' from memory, use the Unload event.
    ''' </remarks>
    Public Overrides Sub ProcessAfterWrite()
        Console.WriteLine("The AfterWrite event of the item has occured. Do not access item's properties in this event!")
    End Sub
 
End Class

